<?php
/**
 * Order Validation Controller
 *
 * @author Andresa Martins <contact@andresa.dev>
 * @license https://opensource.org/licenses/afl-3.0.php
 */
// include _PS_MODULE_DIR_.'plinkpaymentgateway/helper.php';

class PlinkPaymentgatewayValidationModuleFrontController extends ModuleFrontController
{
    private $module_name="plinkpaymentgateway";
    private const MINIMUM_AMOUNT=10000;

    public function postProcess()
    {
        /**
         * Get current cart object from session
         */
        $cart = $this->context->cart;
        $authorized = false;
        $helper = new PlinkPaymentGatewayHelper($this);
        $link = new Link();

        // return dump($helper->isAllConfigurationHasSet());
        /**
         * Verify if this module is enabled and if the cart has
         * a valid customer, delivery address and invoice address
         */
        if (!$this->module->active || $cart->id_customer == 0 || $cart->id_address_delivery == 0
            || $cart->id_address_invoice == 0) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        /**
         * Verify if this payment module is authorized
         */
        foreach (Module::getPaymentModules() as $module) {
            if ($module['name'] == $this->module_name) {
                $authorized = true;
                break;
            }
        }

        if (!$authorized) {
            die($this->l('This payment method is not available.'));
        }

        $cartDetails=$this->context->cart->getSummaryDetails();
        $cartAddressData=$cartDetails['formattedAddresses']['delivery']['object'];
        
        $customer = new Customer($cart->id_customer);
 
        /**
         * Check if this is a vaild customer account
         */
        if (!Validate::isLoadedObject($customer)) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        $err = null;
        
        /**
         * if amount not more than minimum amount
         */
        if (intval(ceil($this->context->cart->getOrderTotal(true, Cart::BOTH))) < 
            self::MINIMUM_AMOUNT) {
            $err = array(
                'msg' => 'The minimum amount for using this payment is IDR ' .Tools::displayPrice(self::MINIMUM_AMOUNT). '. Please put more item(s) to reach the minimum amount. ',
                'btns' => array(
                    array(
                        'txt_btn' => 'Add more items',
                        'url' =>  $link->getPageLink('cart').'?action=show',
                        'icon'=> 'cart-plus'
                    ),
                    array(
                        'txt_btn' => 'Continue Shopping',
                        'url' =>  _PS_BASE_URL_.__PS_BASE_URI__,
                        'icon'=> 'store'
                    )
                )
            );
        }

        // if(!$helper->isCookieEnabled()){
        //     $err = array(
        //         'msg' => 'Cookie is disabled, please enable coookie in your browser',
        //         'btns' => array(
        //             array(
        //                 'txt_btn' => 'Add more items',
        //                 'url' =>  $link->getPageLink('cart').'?action=show',
        //                 'icon'=> 'cart-plus'
        //             ),
        //             array(
        //                 'txt_btn' => 'Continue Shopping',
        //                 'url' =>  _PS_BASE_URL_.__PS_BASE_URI__,
        //                 'icon'=> 'store'
        //             )
        //         )
        //     );
        // }

        /**
         * Check if all configuration exists or valid
         */

        $isAllConfigSet = $helper->isAllConfigurationHasSet();
        $error_config_messages = '';

        if(!$isAllConfigSet->status){
            $i=0;
            foreach($isAllConfigSet->errors as $value){
                if($i < (count($isAllConfigSet->errors)-1)){
                    $error_config_messages .= $value.'&lt;br&gt;';
                }else{
                    $error_config_messages .= $value;
                }

                $i++;   
            }

            $err = array(
                'msg' => '<font >'.$error_config_messages,
                'btns' => array(
                    array(
                        'txt_btn' => 'Retry',
                        'url' =>  'index.php?controller=order&step=1',
                        'icon'=> 'undo'
                    ),
                    array(
                        'txt_btn' => 'Continue Shopping',
                        'url' =>  _PS_BASE_URL_.__PS_BASE_URI__,
                        'icon'=> 'store'
                    )
                )
            );
        }
        
        /**
         * is phone number valid
         */
        if(!preg_match("/^\+[0-9]+/",$cartAddressData['phone'])&&
           !preg_match("/^\+[0-9]+/",$cartAddressData['phone_mobile'])){ 

            $err=array(
                'msg'=>"Phone number must starts with +",
                'btns'=> array(
                    array(
                        'txt_btn' => 'Retry',
                        'url' =>  'index.php?controller=order&step=1',
                        'icon'=> 'undo'
                    ),
                    array(
                        'txt_btn' => 'Continue Shopping',
                        'url' =>  _PS_BASE_URL_.__PS_BASE_URI__,
                        'icon'=> 'store'
                    )
                )
            ); 
        }

        /**
         * If error message exists
         */
        if($err){

            $this->context->smarty->assign(array(
                'buttons' =>$err['btns'],
                'img_logo' => Media::getMediaPath(_PS_MODULE_DIR_.$this->module_name.'/assets/img/plink-logo.png'),
                'msg'=> $err['msg']
            ));
            
            return $this->setTemplate('module:'.$this->module_name.'/views/templates/module/plink_dialogpage.tpl');
        }

        

        /**
         * Place the order
         */
        $this->module->validateOrder(
            (int) $this->context->cart->id,
            Configuration::get('PS_OS_PAYMENT'),
            (float) $this->context->cart->getOrderTotal(true, Cart::BOTH),
            $this->module->displayName,
            null,
            null,
            (int) $this->context->currency->id,
            false,
            $customer->secure_key
        );
        
        /**
        * Redirect the customer to the order confirmation page
        */
        $this->context->cookie->__set('order_id',$this->module->currentOrder);
        $this->context->cookie->write();
        Tools::redirect('index.php?controller=order-confirmation&id_cart='.(int)$cart->id.'&id_module='.(int)$this->module->id.'&id_order='.$this->module->currentOrder.'&key='.$customer->secure_key);

    }
}