
<?php

if (!defined('_PS_VERSION_')) {
    exit;
}

include _PS_MODULE_DIR_.'plinkpaymentgateway/helper.php';

class PlinkPaymentGateway extends PaymentModule
{
    private $_html = '';
    private $_postErrors = array();
    public $address;
    private const PLINK_ORDER_TABLE="plink_pg";
    private $config_keys;
    private $hooks;
    private $plinkHelper;


    public function __construct(){
        $this->name                   = 'plinkpaymentgateway';
        $this->tab                    = 'payments_gateways';
        $this->version                = '1.0';
        $this->author                 = 'Prismalink International';
        $this->controllers            = array('payment', 'validation','payment_flag');
        $this->currencies             = true;
        $this->currencies_mode        = 'checkbox';
        $this->bootstrap              = true;
        $this->displayName            = 'PLINK Payment Gateway';
        $this->description            = 'Payment Gateway (VA, CC, Direct Debit)';
        $this->confirmUninstall       = 'Are you sure you want to uninstall this module?';
        $this->ps_versions_compliancy = array('min' => '1.7.0', 'max' => _PS_VERSION_);
        $this->config_keys            = ['CONF_PLINKPAYMENTGATEWAY_MERCHANTCODE','CONF_PLINKPAYMENTGATEWAY_SECRETKEY','CONF_PLINKPAYMENTGATEWAY_MERCHANTKEYID','CONF_PLINKPAYMENTGATEWAY_MODE','CONF_PLINKPAYMENTGATEWAY_PLINKPGSUCCESS'
        // ,'CONF_PLINKPAYMENTGATEWAY_PLINKPGPENDING','CONF_PLINKPAYMENTGATEWAY_PLINKPGWAITING','CONF_PLINKPAYMENTGATEWAY_PLINKPGREJECT'
        ];
        $this->hooks                  = ['paymentOptions','paymentReturn']; 

        $this->plinkHelper = new PlinkPaymentGatewayHelper($this);

        parent::__construct();
    }
 
    public function install(){

        if (!parent::install() || !$this->registerHook('paymentOptions') || !$this->registerHook('paymentReturn') || !$this->initTable()){
            return false;
        }

        //Register plink payment status
        $languages = Language::getLanguages();

        if (!Configuration::get('CONF_PLINKPAYMENTGATEWAY_PLINKPGSUCCESS')) {
            $orderState = new OrderState();
            $orderState->name = array();
            foreach ($languages as $language) {
                $orderState->name[$language['id_lang']] = 'Plink Payment Gateway - Payment Succeeded';
            }
            $orderState->send_email = true;
            $orderState->module_name = $this->name;
            $orderState->color = '#abd69c';
            $orderState->hidden = false;
            $orderState->delivery = false;
            $orderState->logable = true;
            $orderState->invoice = true;
            if ($orderState->add()) {
                if (copy(dirname(__FILE__) . '/logo.png', dirname(__FILE__) . '/../../img/os/' . (int) $orderState->id . '.gif'))
                    Configuration::updateValue('CONF_PLINKPAYMENTGATEWAY_PLINKPGSUCCESS', (int) $orderState->id);
            }
        }

        if (!Configuration::get('CONF_PLINKPAYMENTGATEWAY_PLINKPGPENDING')) {
            $orderState = new OrderState();
            $orderState->name = array();
            foreach ($languages as $language) {
                $orderState->name[$language['id_lang']] = 'Plink Payment Gateway - Pending Payment';
            }
            $orderState->send_email = true;
            $orderState->module_name = $this->name;
            $orderState->color = '#f9d947';
            $orderState->hidden = false;
            $orderState->delivery = false;
            $orderState->logable = true;
            $orderState->invoice = false;
            if ($orderState->add()) {
                if (copy(dirname(__FILE__) . '/logo.png', dirname(__FILE__) . '/../../img/os/' . (int) $orderState->id . '.gif'))
                    Configuration::updateValue('CONF_PLINKPAYMENTGATEWAY_PLINKPGPENDING', (int) $orderState->id);
            }
        }
        
        if (!Configuration::get('CONF_PLINKPAYMENTGATEWAY_PLINKPGWAITING')) {
            $orderState = new OrderState();
            $orderState->name = array();
            foreach ($languages as $language) {
                $orderState->name[$language['id_lang']] = 'Plink Payment Gateway - Awaiting Payment';
            }
            $orderState->send_email = true;
            $orderState->module_name = $this->name;
            $orderState->color = '#6277e7';
            $orderState->hidden = false;
            $orderState->delivery = false;
            $orderState->logable = true;
            $orderState->invoice = false;
            if ($orderState->add()) {
                if (copy(dirname(__FILE__) . '/logo.png', dirname(__FILE__) . '/../../img/os/' . (int) $orderState->id . '.gif'))
                    Configuration::updateValue('CONF_PLINKPAYMENTGATEWAY_PLINKPGWAITING', (int) $orderState->id);
            }
        }

        if (!Configuration::get('CONF_PLINKPAYMENTGATEWAY_PLINKPGREJECT')) {
            $orderState = new OrderState();
            $orderState->name = array();
            foreach ($languages as $language) {
                $orderState->name[$language['id_lang']] = 'Plink Payment Gateway - Payment Rejected';
            }
            $orderState->send_email = true;
            $orderState->module_name = $this->name;
            $orderState->color = '#ff4747';
            $orderState->hidden = false;
            $orderState->delivery = false;
            $orderState->logable = true;
            $orderState->invoice = false;
            if ($orderState->add()) {
                if (copy(dirname(__FILE__) . '/logo.png', dirname(__FILE__) . '/../../img/os/' . (int) $orderState->id . '.gif'))
                    Configuration::updateValue('CONF_PLINKPAYMENTGATEWAY_PLINKPGREJECT', (int) $orderState->id);
            }
        }

        return true;

    }
 
    public function uninstall(){

        if (!parent::uninstall() || !$this->dropTable()){
            return false;
        }

        // $configs = Configuration::get('MT_ORDER_STATE_ID');
        // $order_state = new OrderStateCore($configs);
        // $order_state->delete();
        
        foreach ($this->hooks as $hook) {
            $this->unregisterHook($hook);
        }

        foreach ($this->config_keys as $key) {
            Configuration::deleteByName($key);
        }

        return true;
    }
    
    public function renderForm(){
        
        $frontEndUrl = Context::getContext()->shop->getBaseURL(true);
        $backeEndUrl = $this->context->link->getModuleLink('plinkpaymentgateway','payment_flag');
        $orderStatuses = OrderState::getOrderStates($this->context->language->id);
        
        $fields_form = array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->l('Settings'),
                    ),
                    'input' => array(
                        array(
                            'type' => 'text',
                            'label' => $this->l('Merchant Code'),
                            'desc' => $this->l('Merchant Code givent by PLINK'),
                            'name' => 'merchantCode',
                            'required' => true
                        ),
                        array(
                            'type' => 'text',
                            'label' => $this->l('Secret Key'),
                            'desc' => $this->l('Secret Key given by PLINK'),
                            'name' => 'secretKey',
                            'required' => true
                        ),
                        array(
                            'type' => 'text',
                            'label' => $this->l('Merchant Key ID'),
                            'desc' => $this->l('Merchant Key ID given by PLINK'),
                            'name' => 'merchantKeyId',
                            'required' => true
                        ),
                        array(
                            'type' => 'select',
                            'label' => $this->l('Mode'),
                            'name' => 'mode',
                            'required' => true,
                            'desc' => $this->l('Payment mode: testing mode doesn\'t require real payment and live mode is real payment mode'),
                            'options' => array(
                                'query' => array(
                                    array(
                                        'id' => 'test',
                                        'name' => 'Test',
                                        'selected' => true
                                    ),
                                    array(
                                        'id' => 'live',
                                        'name' => 'Live')
                                ),
                                'id' => 'id',
                                'name' => 'name'
                            )
                        )
                        , 
                        array(
                            'type' => 'select',
                            'label' => $this->l('Payment Status - Pending'),
                            'name' => 'plinkpgPending',
                            'desc' => $this->l('Map on pending for payment status'),
                            'required' => true,
                            'options' =>array(
                                'query' => $orderStatuses,
                                'id' => 'id_order_state',
                                'name' => 'name'
                            )
                        ),
                        array(
                            'type' => 'select',
                            'label' => $this->l('Payment Status - Waiting'),
                            'name' => 'plinkpgWaiting',
                            'desc' => $this->l('Map on waiting for payment status'),
                            'required' => true,
                            'options' =>array(
                                'query' => $orderStatuses,
                                'id' => 'id_order_state',
                                'name' => 'name'
                            )
                        ),
                        array(
                            'type' => 'select',
                            'label' => $this->l('Payment Status - Success'),
                            'name' => 'plinkpgSuccess',
                            'required' => true,
                            'desc' => $this->l('Map on success payment status'),
                            'options' =>array(
                                'query' => $orderStatuses,
                                'id' => 'id_order_state',
                                'name' => 'name'
                            )
                        ),
                        array(
                            'type' => 'select',
                            'label' => $this->l('Payment Status - Rejected'),
                            'name' => 'plinkpgRejected',
                            'desc' => $this->l('Map on rejected payment status'),
                            'required' => true,
                            'options' =>array(
                                'query' => $orderStatuses,
                                'id' => 'id_order_state',
                                'name' => 'name'
                            )
                        )
                    ),
                    'submit' => array(
                        'title' => $this->l('Save'),
                        'class' => 'button'
                    ),
                ),
        );

        $fields_form3 = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Callback Urls'),
                ),
                'input' => array(
                    array(
                        'type' => 'text',
                        'label' => 'Backend Callback URL',
                        'name' => 'backendCbUrl',
                        // 'readonly' => true,
                        'value' => $backeEndUrl,
                        'desc' => $this->l('Url to confirm payment status. Used by Plink backend')
                    ),
                    array(
                        'type' => 'text',
                        'label' => 'Frontend Callback URL',
                        'name' => 'frontendCbUrl',
                        // 'readonly' => true,
                        'value' => $frontEndUrl,
                        'desc' => $this->l('Url to confirm payment status. Used by Plink backend')
                    )
                ),
            )
        );

        $helper = new HelperForm();

        $helper->module             = $this;
        $helper->name_controller    = $this->name; //requires the name of the module.
        $helper->token              = Tools::getAdminTokenLite('AdminModules'); //requires a unique token for the module. getAdminTokenLite() helps us generate one
        $helper->currentIndex       = AdminController::$currentIndex . '&configure=' . $this->name;

        $default_lang               = (int) Configuration::get('PS_LANG_DEFAULT'); //
        $helper->default_form_language      = $default_lang; //requires the default language for the shop.
        $helper->allow_employee_form_lang   = $default_lang; //requires the default language for the shop.

        $helper->title              = $this->displayName;
        $helper->show_toolbar       = true;        // false -> remove toolbar
        $helper->toolbar_scroll     = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action      = 'btnSubmit' . $this->name;
        
        $helper->toolbar_btn        = array(
           'save' => array(
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name .
                '&token=' . Tools::getAdminTokenLite('AdminModules'),
            ),
            'back' => array(
                'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );
        
        $dataForm=['merchantCode','secretKey','merchantKeyId','mode'];

        foreach($dataForm as $value){
            $helper->fields_value[$value]  = Configuration::get('CONF_PLINKPAYMENTGATEWAY_'.strtoupper($value));    
        }
        
        $paymentStatus = [
            'plinkpgSuccess' => 'CONF_PLINKPAYMENTGATEWAY_PLINKPGSUCCESS',
            'plinkpgPending' => 'CONF_PLINKPAYMENTGATEWAY_PLINKPGPENDING',
            'plinkpgWaiting' => 'CONF_PLINKPAYMENTGATEWAY_PLINKPGWAITING',
            'plinkpgRejected' => 'CONF_PLINKPAYMENTGATEWAY_PLINKPGREJECT'];

        foreach($paymentStatus as $key => $value){
            // return dump(Configuration::get('CONF_PLINKPAYMENTGATEWAY_'.$value));
            $helper->fields_value[$key] = Configuration::get($value);
        }

        $helper->fields_value['backendCbUrl']  = $backeEndUrl;    
        $helper->fields_value['frontendCbUrl']  = $frontEndUrl;

        return $helper->generateForm(array($fields_form,$fields_form3));
    }

    public function getContent(){

        if (Tools::isSubmit('btnSubmit'.$this->name)) {
            
            $this->_postValidation();

            if (!count($this->_postErrors))
                $this->_postProcess();
            else
                foreach ($this->_postErrors as $err)
                    $this->_html .= $this->displayError($err);

        } else
            $this->_html .= '<br />';

        $this->_html .= $this->renderForm();

        return $this->_html;
    }

    protected function _postValidation(){

        if (Tools::isSubmit('btnSubmit' . $this->name)) {

            $dataForm=array("merchantCode" => "Merchant Code",
                            "secretKey" => "Secret Key",
                            "merchantKeyId" => "Merchant Key ID",
                            "mode" => "Mode",
                            "plinkpgSuccess" => "Plink Success Status",
                            "plinkpgPending" => "Plink Pending Status",
                            "plinkpgWaiting" => "Plink Waiting Status",
                            "plinkpgRejected" => "Plink Rejected Status"
                        );
            
            foreach($dataForm as $key => $value){
                
                $form = Tools::getValue($key);
           
                if(!$form || !Validate::isGenericName($form))
                    $this->_postErrors[] = $this->l('Invalid '.$value.' value.'); 
               
            }
        }
    }

    protected function _postProcess(){

        $dataForm=['merchantCode','secretKey','merchantKeyId','mode','plinkpgSuccess','plinkpgPending','plinkpgWaiting','plinkpgRejected'];

        if (Tools::isSubmit('btnSubmit' . $this->name)) { 
          
            foreach($dataForm as $value){
                Configuration::updateValue('CONF_PLINKPAYMENTGATEWAY_'.strtoupper($value),Tools::getValue($value));
            }

            $this->_html .= $this->displayConfirmation($this->l('Updated Succesfull'));
        }
    }

    private function initTable(){

        $sql="CREATE TABLE IF NOT EXISTS "._DB_PREFIX_.self::PLINK_ORDER_TABLE." (
            `order_id` INT(255) NOT NULL,
            `plink_payment_status` varchar(10),
            `user_id` varchar(255) ,
            `plink_ref_no` varchar(255),
            `email` varchar(255),
            `order_created` varchar(255),
            `payment_date` varchar(255),
            `description` varchar(255),
            PRIMARY KEY ( order_id ))
        ";

        return Db::getInstance()->execute($sql);
    
    }

    private function dropTable(){
        $sql="DROP TABLE IF EXISTS "._DB_PREFIX_.self::PLINK_ORDER_TABLE;
        return Db::getInstance()->execute($sql);
    }

   
    ///////////////////////FRONT PAGE////////////////
   
    /**
     * Add payment options on checkout page
     */
    public function hookPaymentOptions($params){
 
        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }
 
        $formAction = $this->context->link->getModuleLink($this->name, 'validation', array(), true);
        
        $this->smarty->assign([
            'action' => $formAction ,
        ]);
 
        
        
        $paymentForm = $this->fetch('module:'.$this->name.'/views/templates/hook/payment_options.tpl');
 
        /**
         * Create a PaymentOption object containing the necessary data
         * to display this module in the checkout
         */
        $newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption;
        $newOption->setModuleName($this->displayName)
            ->setCallToActionText($this->displayName)
            ->setAction($formAction)
            ->setForm($paymentForm)
            ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/assets/img/plink-logo.png'));
 
        $payment_options = array(
            $newOption
        );
 
        return $payment_options;
    }

    public function checkCurrency($cart) {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module))
            foreach ($currencies_module as $currency_module)
                if ($currency_order->id == $currency_module['id_currency'] && $currency_order->iso_code == "IDR") {
                    return true;
                }
                
        return false;
    }

    /**
     * Display payment message on payment return
     */
    public function hookPaymentReturn($params){

        /**
         * Verify if this module is enabled
         */
        
        if (!$this->active) {
            return;
        }

        $order=$params['order'];

        $msgs = array();

        if(Configuration::get('CONF_PLINKPAYMENTGATEWAY_MODE') === 'test'){
            array_push($msgs,array('color'=>'red','msg'=>"TEST MODE - Provided virtual account numbers is for testing. Real payment will not be detected"));
        }

        $message_head='Congratulation!';
        $message_content='You have just completed an order using PLINK Payment Gateway.';
        

        $this->smarty->assign([
            'msg_head'=>$message_head,
            'msg_content'=>$message_content,
            'order_id' => $order->id,
            'msg'=> $msgs  
        ]);
        
        return $this->fetch('module:'.$this->name.'/views/templates/hook/payment_return.tpl');
    }
  
}